searchData={"items":[{"type":"module","title":"Pigeon","doc":"HTTP2-compliant wrapper for sending iOS and Android push notifications.","ref":"Pigeon.html"},{"type":"function","title":"Pigeon.debug_log?/0","doc":"","ref":"Pigeon.html#debug_log?/0"},{"type":"function","title":"Pigeon.json_library/0","doc":"Returns the configured JSON encoding library for Pigeon.\nTo customize the JSON library, include the following in your config/config.exs:\n\n    config :pigeon, :json_library, Jason","ref":"Pigeon.html#json_library/0"},{"type":"module","title":"Pigeon.ADM","doc":"Amazon Device Messaging (ADM)","ref":"Pigeon.ADM.html"},{"type":"function","title":"Pigeon.ADM.push/2","doc":"Sends a push over ADM.","ref":"Pigeon.ADM.html#push/2"},{"type":"function","title":"Examples - Pigeon.ADM.push/2","doc":"iex> msg = %{\"body\" => \"your message\"}\n    iex> n = Pigeon.ADM.Notification.new(\"your_reg_id\", msg)\n    iex> Pigeon.ADM.push(n, on_response: nil)\n    :ok\n\n    iex> msg = %{\"body\" => \"your message\"}\n    iex> n = Pigeon.ADM.Notification.new(\"your_reg_id\", msg)\n    iex> Pigeon.ADM.push(n)\n    %Pigeon.ADM.Notification{consolidation_key: nil,\n     expires_after: 604800, md5: \"M13RuG4uDWqajseQcCiyiw==\",\n     payload: %{\"data\" => %{\"body\" => \"your message\"}},\n     registration_id: \"your_reg_id\", response: :invalid_registration_id,\n     updated_registration_id: nil}\n\n    iex> msg = %{\"body\" => \"your message\"}\n    iex> n = Pigeon.ADM.Notification.new(\"your_reg_id\", msg)\n    iex> notifs = Pigeon.ADM.push([n, n], to: :adm_default)\n    iex> Enum.map(notifs, & &1.response)\n    [:invalid_registration_id, :invalid_registration_id]\n\n    iex> me = self()\n    iex> handler = fn(_x) -> send(me, \"Sent a push!\") end\n    iex> n = Pigeon.ADM.Notification.new(\"your_reg_id\", %{})\n    iex> Pigeon.ADM.push(n, on_response: handler)\n    iex> receive do\n    ...>   x -> x\n    ...> after\n    ...>   5_000 -> \"No push response...\"\n    ...> end\n    \"Sent a push!\"\n\n    iex> msg = %{\"body\" => \"your message\"}\n    iex> n = Pigeon.ADM.Notification.new(\"your_reg_id\", msg)\n    iex> notif = Pigeon.ADM.push(n, to: :worker_not_started)\n    iex> notif.response\n    :timeout","ref":"Pigeon.ADM.html#push/2-examples"},{"type":"function","title":"Pigeon.ADM.start_connection/1","doc":"Starts ADM worker connection with given config or name.","ref":"Pigeon.ADM.html#start_connection/1"},{"type":"function","title":"Examples - Pigeon.ADM.start_connection/1","doc":"iex> config = Pigeon.ADM.Config.new(:adm_default)\n    iex> {:ok, pid} = Pigeon.ADM.start_connection(%{config | name: nil})\n    iex> Process.alive?(pid)\n    true","ref":"Pigeon.ADM.html#start_connection/1-examples"},{"type":"function","title":"Pigeon.ADM.stop_connection/1","doc":"Stops existing ADM worker connection.","ref":"Pigeon.ADM.html#stop_connection/1"},{"type":"function","title":"Examples - Pigeon.ADM.stop_connection/1","doc":"iex> config = Pigeon.ADM.Config.new(:adm_default)\n    iex> {:ok, pid} = Pigeon.ADM.start_connection(%{config | name: nil})\n    iex> Pigeon.ADM.stop_connection(pid)\n    :ok\n    iex> :timer.sleep(500)\n    iex> Process.alive?(pid)\n    false","ref":"Pigeon.ADM.html#stop_connection/1-examples"},{"type":"type","title":"Pigeon.ADM.connection_response/0","doc":"","ref":"Pigeon.ADM.html#t:connection_response/0"},{"type":"type","title":"Pigeon.ADM.on_response/0","doc":"Async callback for push notifications response.","ref":"Pigeon.ADM.html#t:on_response/0"},{"type":"type","title":"Examples - Pigeon.ADM.on_response/0","doc":"handler = fn(%Pigeon.ADM.Notification{response: response}) ->\n      case response do\n        :success ->\n          Logger.debug \"Push successful!\"\n        :unregistered ->\n          Logger.error \"Bad device token!\"\n        _error ->\n          Logger.error \"Some other error happened.\"\n      end\n    end\n\n    n = Pigeon.ADM.Notification.new(\"token\", %{\"message\" => \"test\"})\n    Pigeon.ADM.push(n, on_response: handler)","ref":"Pigeon.ADM.html#t:on_response/0-examples"},{"type":"type","title":"Pigeon.ADM.push_opts/0","doc":"Options for sending push notifications.\n\n- `:to` - Defines worker to process push. Defaults to `:adm_default`\n- `:on_response` - Optional async callback triggered on receipt of push.\n  See `t:on_response/0`","ref":"Pigeon.ADM.html#t:push_opts/0"},{"type":"module","title":"Pigeon.ADM.Config","doc":"Validates configuration settings that initialize ADM.Worker instances.","ref":"Pigeon.ADM.Config.html"},{"type":"function","title":"Pigeon.ADM.Config.default_name/0","doc":"","ref":"Pigeon.ADM.Config.html#default_name/0"},{"type":"function","title":"Pigeon.ADM.Config.new/1","doc":"Returns a new `ADM.Config` with given `opts`.","ref":"Pigeon.ADM.Config.html#new/1"},{"type":"function","title":"Examples - Pigeon.ADM.Config.new/1","doc":"iex> Pigeon.ADM.Config.new(\n    ...>   name: :test,\n    ...>   client_id: \"amzn.client.id\",\n    ...>   client_secret: \"1234secret\"\n    ...> )\n    %Pigeon.ADM.Config{name: :test, client_id: \"amzn.client.id\",\n    client_secret: \"1234secret\"}","ref":"Pigeon.ADM.Config.html#new/1-examples"},{"type":"function","title":"Pigeon.ADM.Config.valid?/1","doc":"Returns whether a given config has valid credentials.","ref":"Pigeon.ADM.Config.html#valid?/1"},{"type":"function","title":"Examples - Pigeon.ADM.Config.valid?/1","doc":"iex> :adm_default |> new() |> valid?()\n    true","ref":"Pigeon.ADM.Config.html#valid?/1-examples"},{"type":"function","title":"Pigeon.ADM.Config.validate!/1","doc":"","ref":"Pigeon.ADM.Config.html#validate!/1"},{"type":"type","title":"Pigeon.ADM.Config.t/0","doc":"","ref":"Pigeon.ADM.Config.html#t:t/0"},{"type":"module","title":"Pigeon.ADM.Notification","doc":"Defines Amazon ADM notification struct and convenience constructor functions.","ref":"Pigeon.ADM.Notification.html"},{"type":"function","title":"Pigeon.ADM.Notification.new/2","doc":"Creates `ADM.Notification` struct with device registration ID and optional data payload.","ref":"Pigeon.ADM.Notification.html#new/2"},{"type":"function","title":"Examples - Pigeon.ADM.Notification.new/2","doc":"iex> Pigeon.ADM.Notification.new(\"reg ID\")\n    %Pigeon.ADM.Notification{\n      consolidation_key: nil,\n      md5: \"1B2M2Y8AsgTpgAmY7PhCfg==\",\n      payload: %{\"data\" => %{}},\n      registration_id: \"reg ID\",\n      updated_registration_id: nil\n    }\n\n    iex> Pigeon.ADM.Notification.new(\"reg ID\", %{\"message\" => \"your message\"})\n    %Pigeon.ADM.Notification{\n      consolidation_key: nil,\n      md5: \"qzF+HgArKZjJrpfcTbiFxg==\",\n      payload: %{\n        \"data\" => %{\"message\" => \"your message\"}\n      },\n      registration_id: \"reg ID\",\n      updated_registration_id: nil\n    }\n\n    iex> Pigeon.ADM.Notification.new(\"reg ID\", \"not a map\")\n    %Pigeon.ADM.Notification{\n      consolidation_key: nil,\n      md5: \"1B2M2Y8AsgTpgAmY7PhCfg==\",\n      payload: %{\"data\" => %{}},\n      registration_id: \"reg ID\",\n      updated_registration_id: nil\n    }","ref":"Pigeon.ADM.Notification.html#new/2-examples"},{"type":"function","title":"Pigeon.ADM.Notification.put_data/2","doc":"Updates `\"data\"` key on push payload and calculates `md5` hash.","ref":"Pigeon.ADM.Notification.html#put_data/2"},{"type":"function","title":"Examples - Pigeon.ADM.Notification.put_data/2","doc":"iex> n = %Pigeon.ADM.Notification{}\n    iex> Pigeon.ADM.Notification.put_data(n, %{\"message\" => \"your message\"})\n    %Pigeon.ADM.Notification{\n      consolidation_key: nil,\n      md5: \"qzF+HgArKZjJrpfcTbiFxg==\",\n      payload: %{\n        \"data\" => %{\"message\" => \"your message\"}\n      },\n      registration_id: nil,\n      updated_registration_id: nil\n    }","ref":"Pigeon.ADM.Notification.html#put_data/2-examples"},{"type":"type","title":"Pigeon.ADM.Notification.error_response/0","doc":"ADM error responses","ref":"Pigeon.ADM.Notification.html#t:error_response/0"},{"type":"type","title":"Pigeon.ADM.Notification.response/0","doc":"ADM push response\n\n- nil - Push has not been sent yet\n- `:success` - Push was successfully sent\n- `t:error_response/0` - Push attempted but server responded\n  with error\n- `:timeout` - Internal error. Push did not reach ADM servers","ref":"Pigeon.ADM.Notification.html#t:response/0"},{"type":"type","title":"Pigeon.ADM.Notification.t/0","doc":"ADM notification","ref":"Pigeon.ADM.Notification.html#t:t/0"},{"type":"type","title":"Examples - Pigeon.ADM.Notification.t/0","doc":"%Pigeon.ADM.Notification{\n      consolidation_key: nil,\n      expires_after: 604_800,\n      md5: \"qzF+HgArKZjJrpfcTbiFxg==\",\n      payload: %{\n        \"data\" => %{\"message\" => \"your message\"}\n      },\n      registration_id: \"reg ID\",\n      response: nil, # Set on push response\n      updated_registration_id: nil\n    }","ref":"Pigeon.ADM.Notification.html#t:t/0-examples"},{"type":"module","title":"Pigeon.APNS","doc":"Apple Push Notification Service (APNS)","ref":"Pigeon.APNS.html"},{"type":"function","title":"Pigeon.APNS.push/2","doc":"Sends a push over APNS.","ref":"Pigeon.APNS.html#push/2"},{"type":"function","title":"Examples - Pigeon.APNS.push/2","doc":"iex> n = Pigeon.APNS.Notification.new(\"msg\", \"token\", \"topic\")\n     iex> Pigeon.APNS.push(n)\n     %Pigeon.APNS.Notification{device_token: \"token\", expiration: nil,\n      response: :bad_device_token, id: nil,\n      payload: %{\"aps\" => %{\"alert\" => \"msg\"}}, topic: \"topic\"}\n\n     iex> n = Pigeon.APNS.Notification.new(\"msg\", \"token\", \"topic\")\n     iex> Pigeon.APNS.push([n, n], on_response: nil)\n     :ok\n\n     iex> n = Pigeon.APNS.Notification.new(\"msg\", \"token\", \"topic\")\n     iex> Pigeon.APNS.push([n, n])\n     [%Pigeon.APNS.Notification{device_token: \"token\", expiration: nil,\n       response: :bad_device_token, id: nil,\n       payload: %{\"aps\" => %{\"alert\" => \"msg\"}}, topic: \"topic\"},\n      %Pigeon.APNS.Notification{device_token: \"token\", expiration: nil,\n       response: :bad_device_token, id: nil,\n       payload: %{\"aps\" => %{\"alert\" => \"msg\"}}, topic: \"topic\"}]","ref":"Pigeon.APNS.html#push/2-examples"},{"type":"function","title":"Pigeon.APNS.start_connection/1","doc":"Starts APNS worker connection with given config or name.","ref":"Pigeon.APNS.html#start_connection/1"},{"type":"function","title":"Examples - Pigeon.APNS.start_connection/1","doc":"iex> config = Pigeon.APNS.Config.new(:apns_default)\n    iex> {:ok, pid} = Pigeon.APNS.start_connection(%{config | name: nil})\n    iex> Process.alive?(pid)\n    true","ref":"Pigeon.APNS.html#start_connection/1-examples"},{"type":"function","title":"Pigeon.APNS.stop_connection/1","doc":"Stops existing APNS worker connection.","ref":"Pigeon.APNS.html#stop_connection/1"},{"type":"function","title":"Examples - Pigeon.APNS.stop_connection/1","doc":"iex> config = Pigeon.APNS.Config.new(:apns_default)\n    iex> {:ok, pid} = Pigeon.APNS.start_connection(%{config | name: nil})\n    iex> Pigeon.APNS.stop_connection(pid)\n    :ok\n    iex> :timer.sleep(500)\n    iex> Process.alive?(pid)\n    false","ref":"Pigeon.APNS.html#stop_connection/1-examples"},{"type":"type","title":"Pigeon.APNS.notification/0","doc":"Can be either a single notification or a list.","ref":"Pigeon.APNS.html#t:notification/0"},{"type":"type","title":"Pigeon.APNS.on_response/0","doc":"Async callback for push notification response.","ref":"Pigeon.APNS.html#t:on_response/0"},{"type":"type","title":"Examples - Pigeon.APNS.on_response/0","doc":"handler = fn(%Pigeon.APNS.Notification{response: response}) ->\n      case response do\n        :success ->\n          Logger.debug \"Push successful!\"\n        :bad_device_token ->\n          Logger.error \"Bad device token!\"\n        _error ->\n          Logger.error \"Some other error happened.\"\n      end\n    end\n\n    n = Pigeon.APNS.Notification.new(\"msg\", \"device token\", \"push topic\")\n    Pigeon.APNS.push(n, on_response: handler)","ref":"Pigeon.APNS.html#t:on_response/0-examples"},{"type":"type","title":"Pigeon.APNS.push_opts/0","doc":"Options for sending push notifications.\n\n- `:to` - Defines worker to process push. Defaults to `:apns_default`\n- `:on_response` - Optional async callback triggered on receipt of push.\n  See `t:on_response/0`","ref":"Pigeon.APNS.html#t:push_opts/0"},{"type":"module","title":"Pigeon.APNS.Config","doc":"Configuration for APNS Workers using certificates.","ref":"Pigeon.APNS.Config.html"},{"type":"function","title":"Pigeon.APNS.Config.new/1","doc":"Returns a new `APNS.Config` with given `opts` or name.\n\nIf given an atom, returns the config specified in your `config.exs`.","ref":"Pigeon.APNS.Config.html#new/1"},{"type":"function","title":"Examples - Pigeon.APNS.Config.new/1","doc":"iex> Pigeon.APNS.Config.new(\n    ...>   name: :test,\n    ...>   mode: :prod,\n    ...>   cert: \"test/support/test_cert.pem-mock\",\n    ...>   key: \"test/support/test_key.pem-mock\",\n    ...>   port: 2197,\n    ...>   ping_period: 300_000\n    ...> )\n    %Pigeon.APNS.Config{uri: \"api.push.apple.com\", name: :test,\n    certfile: Path.expand(\"test/support/test_cert.pem-mock\"),\n    keyfile: Path.expand(\"test/support/test_key.pem-mock\"),\n    ping_period: 300000, port: 2197, reconnect: false}\n\n    iex> config = Pigeon.APNS.Config.new(:apns_default)\n    iex> %{config | certfile: nil, keyfile: nil} # Hide for testing\n    iex> match? %_{uri: \"api.development.push.apple.com\",\n    ...> name: :apns_default, ping_period: 600_000, port: 443}, config\n    true","ref":"Pigeon.APNS.Config.html#new/1-examples"},{"type":"type","title":"Pigeon.APNS.Config.config_opts/0","doc":"Options for configuring certificate APNS connections.","ref":"Pigeon.APNS.Config.html#t:config_opts/0"},{"type":"type","title":"Configuration Options - Pigeon.APNS.Config.config_opts/0","doc":"- `:name` - Registered worker name.\n- `:mode` - If set to `:dev` or `:prod`, will set the appropriate `:uri`\n- `:cert` - Push certificate. Can be one of three options:\n  - Static file path\n  - Full-text string of the file contents (useful for environment variables)\n  - `{:my_app, \"certs/cert.pem\"}` (indicates path relative to the `priv`\n    folder of the given application)\n- `:key` - Push private key. Same as `:cert`\n- `:uri` - Push server uri. If set, overrides uri defined by `:mode`.\n  Useful for test environments.\n- `:port` - Push server port. Can be any value, but APNS only accepts\n  `443` and `2197`\n- `:ping_period` - Interval between server pings. Necessary to keep long\n  running APNS connections alive. Defaults to 10 minutes.","ref":"Pigeon.APNS.Config.html#t:config_opts/0-configuration-options"},{"type":"type","title":"Deprecated Options - Pigeon.APNS.Config.config_opts/0","doc":"- `:reconnect` - No longer used as of `v1.2.0`.","ref":"Pigeon.APNS.Config.html#t:config_opts/0-deprecated-options"},{"type":"type","title":"Pigeon.APNS.Config.t/0","doc":"Certificate APNS configuration struct\n\nThis struct should not be set directly. Instead use `new/1`\nwith `t:config_opts/0`.","ref":"Pigeon.APNS.Config.html#t:t/0"},{"type":"type","title":"Examples - Pigeon.APNS.Config.t/0","doc":"%Pigeon.APNS.Config{\n      name: :apns_default,\n      reconnect: true,\n      cert: nil,\n      certfile: \"cert.pem\",\n      key: nil,\n      keyfile: \"key.pem\",\n      uri: \"api.push.apple.com\",\n      port: 443,\n      ping_period: 600_000\n    }","ref":"Pigeon.APNS.Config.html#t:t/0-examples"},{"type":"module","title":"Pigeon.APNS.Error","doc":"Defines APNS error responses","ref":"Pigeon.APNS.Error.html"},{"type":"function","title":"Pigeon.APNS.Error.log/2","doc":"If enabled, logs a notification and its error response.","ref":"Pigeon.APNS.Error.html#log/2"},{"type":"type","title":"Pigeon.APNS.Error.error_response/0","doc":"","ref":"Pigeon.APNS.Error.html#t:error_response/0"},{"type":"module","title":"Pigeon.APNS.JWTConfig","doc":"Configuration for APNS Workers using JWT.","ref":"Pigeon.APNS.JWTConfig.html"},{"type":"function","title":"Pigeon.APNS.JWTConfig.new/1","doc":"Returns a new `APNS.JWTConfig` with given `opts` or name.\n\nIf given an atom, returns the config specified in your `config.exs`.","ref":"Pigeon.APNS.JWTConfig.html#new/1"},{"type":"function","title":"Examples - Pigeon.APNS.JWTConfig.new/1","doc":"iex> Pigeon.APNS.JWTConfig.new(\n    ...>   name: :test,\n    ...>   mode: :prod,\n    ...>   key: \"test/support/AuthKey.p8-mock\",\n    ...>   key_identifier: \"ABC1234567\",\n    ...>   team_id: \"DEF1234567\",\n    ...>   port: 2197,\n    ...>   ping_period: 300_000\n    ...> )\n    %Pigeon.APNS.JWTConfig{uri: \"api.push.apple.com\", name: :test,\n    team_id: \"DEF1234567\", key_identifier: \"ABC1234567\", \n    keyfile: Path.expand(\"test/support/AuthKey.p8-mock\"),\n    ping_period: 300000, port: 2197, reconnect: false}\n\n    iex> config = Pigeon.APNS.JWTConfig.new(:apns_jwt_static)\n    iex> %{config | key: nil, key_identifier: nil, team_id: nil} # Hide for testing\n    iex> match? %_{uri: \"api.development.push.apple.com\", name: :apns_jwt_static,\n    ...> ping_period: 600_000, port: 443, reconnect: false}, config\n    true","ref":"Pigeon.APNS.JWTConfig.html#new/1-examples"},{"type":"type","title":"Pigeon.APNS.JWTConfig.config_opts/0","doc":"Options for configuring JWT APNS connections.","ref":"Pigeon.APNS.JWTConfig.html#t:config_opts/0"},{"type":"type","title":"Configuration Options - Pigeon.APNS.JWTConfig.config_opts/0","doc":"- `:name` - Registered worker name.\n- `:mode` - If set to `:dev` or `:prod`, will set the appropriate `:uri`\n- `:key` - JWT private key. Can be one of three options:\n  - Static file path\n  - Full-text string of the file contents (useful for environment variables)\n  - `{:my_app, \"keys/private_key.p8\"}` (indicates path relative to the `priv`\n    folder of the given application)\n- `:key_identifier` - A 10-character key identifier (kid) key, obtained from\n  your developer account\n- `:team_id` - Your 10-character Team ID, obtained from your developer account\n- `:uri` - Push server uri. If set, overrides uri defined by `:mode`.\n  Useful for test environments.\n- `:port` - Push server port. Can be any value, but APNS only accepts\n  `443` and `2197`\n- `:ping_period` - Interval between server pings. Necessary to keep long\n  running APNS connections alive. Defaults to 10 minutes.","ref":"Pigeon.APNS.JWTConfig.html#t:config_opts/0-configuration-options"},{"type":"type","title":"Deprecated Options - Pigeon.APNS.JWTConfig.config_opts/0","doc":"- `:reconnect` - No longer used as of `v1.2.0`.","ref":"Pigeon.APNS.JWTConfig.html#t:config_opts/0-deprecated-options"},{"type":"type","title":"Pigeon.APNS.JWTConfig.headers/0","doc":"","ref":"Pigeon.APNS.JWTConfig.html#t:headers/0"},{"type":"type","title":"Pigeon.APNS.JWTConfig.t/0","doc":"JWT APNS configuration struct\n\nThis struct should not be set directly. Instead use `new/1`\nwith `t:config_opts/0`.","ref":"Pigeon.APNS.JWTConfig.html#t:t/0"},{"type":"type","title":"Examples - Pigeon.APNS.JWTConfig.t/0","doc":"%Pigeon.APNS.JWTConfig{\n      name: :apns_default,\n      reconnect: true,\n      uri: \"api.push.apple.com\",\n      port: 443,\n      ping_period: 600_000,\n      key: nil,\n      keyfile: \"key.p8\",\n      key_identifier: \"ABC1234567\",\n      team_id: \"DEF1234567\"\n    }","ref":"Pigeon.APNS.JWTConfig.html#t:t/0-examples"},{"type":"module","title":"Pigeon.APNS.Notification","doc":"Defines APNS notification struct and constructor functions.","ref":"Pigeon.APNS.Notification.html"},{"type":"function","title":"Pigeon.APNS.Notification.new/3","doc":"Returns an `APNS.Notification` struct with given message, device token, and\ntopic (optional).\n\nPush payload is constructed in the form of `%{\"aps\" => %{\"alert\" => msg}}`","ref":"Pigeon.APNS.Notification.html#new/3"},{"type":"function","title":"Examples - Pigeon.APNS.Notification.new/3","doc":"iex> Pigeon.APNS.Notification.new(\"push message\", \"device token\")\n    %Pigeon.APNS.Notification{\n      device_token: \"device token\",\n      expiration: nil,\n      priority: nil,\n      push_type: \"alert\",\n      id: nil,\n      payload: %{\"aps\" => %{\"alert\" => \"push message\"}},\n      topic: nil\n    }","ref":"Pigeon.APNS.Notification.html#new/3-examples"},{"type":"function","title":"Pigeon.APNS.Notification.new/4","doc":"Returns an `APNS.Notification` struct with given message, device token,\ntopic, and message ID.\n\nPush payload is constructed in the form of `%{\"aps\" => %{\"alert\" => msg}}`","ref":"Pigeon.APNS.Notification.html#new/4"},{"type":"function","title":"Examples - Pigeon.APNS.Notification.new/4","doc":"iex> Pigeon.APNS.Notification.new(\"push message\", \"device token\", \"topic\", \"id_1234\")\n    %Pigeon.APNS.Notification{\n      collapse_id: nil,\n      device_token: \"device token\",\n      expiration: nil,\n      priority: nil,\n      push_type: \"alert\",\n      id: \"id_1234\",\n      payload: %{\"aps\" => %{\"alert\" => \"push message\"}},\n      topic: \"topic\"\n    }","ref":"Pigeon.APNS.Notification.html#new/4-examples"},{"type":"function","title":"Pigeon.APNS.Notification.put_alert/2","doc":"Updates `\"alert\"` key in push payload.\n\nThis is the alert message displayed on the device.","ref":"Pigeon.APNS.Notification.html#put_alert/2"},{"type":"function","title":"Examples - Pigeon.APNS.Notification.put_alert/2","doc":"iex> Pigeon.APNS.Notification.put_alert(%Pigeon.APNS.Notification{}, \"push message\")\n    %Pigeon.APNS.Notification{\n      collapse_id: nil,\n      device_token: nil,\n      expiration: nil,\n      priority: nil,\n      push_type: \"alert\",\n      id: nil,\n      payload: %{\"aps\" => %{\"alert\" => \"push message\"}},\n      topic: nil\n    }","ref":"Pigeon.APNS.Notification.html#put_alert/2-examples"},{"type":"function","title":"Pigeon.APNS.Notification.put_badge/2","doc":"Updates `\"badge\"` key in push payload.\n\nThis is the badge number displayed on the application.","ref":"Pigeon.APNS.Notification.html#put_badge/2"},{"type":"function","title":"Examples - Pigeon.APNS.Notification.put_badge/2","doc":"iex> Pigeon.APNS.Notification.put_badge(%Pigeon.APNS.Notification{}, 5)\n    %Pigeon.APNS.Notification{\n      collapse_id: nil,\n      device_token: nil,\n      expiration: nil,\n      priority: nil,\n      push_type: \"alert\",\n      id: nil,\n      payload: %{\"aps\" => %{\"badge\" => 5}},\n      topic: nil\n    }","ref":"Pigeon.APNS.Notification.html#put_badge/2-examples"},{"type":"function","title":"Pigeon.APNS.Notification.put_category/2","doc":"Updates `\"category\"` key in push payload.","ref":"Pigeon.APNS.Notification.html#put_category/2"},{"type":"function","title":"Examples - Pigeon.APNS.Notification.put_category/2","doc":"iex> Pigeon.APNS.Notification.put_category(%Pigeon.APNS.Notification{}, \"category\")\n    %Pigeon.APNS.Notification{\n      collapse_id: nil,\n      device_token: nil,\n      expiration: nil,\n      priority: nil,\n      push_type: \"alert\",\n      id: nil,\n      payload: %{\"aps\" => %{\"category\" => \"category\"}},\n      topic: nil\n    }","ref":"Pigeon.APNS.Notification.html#put_category/2-examples"},{"type":"function","title":"Pigeon.APNS.Notification.put_content_available/1","doc":"Sets `\"content-available\"` flag in push payload.\n\nUsed for silent notifications. When set, ensure `alert`, `badge`, and `sound` keys\nare not configured.","ref":"Pigeon.APNS.Notification.html#put_content_available/1"},{"type":"function","title":"Examples - Pigeon.APNS.Notification.put_content_available/1","doc":"iex> Pigeon.APNS.Notification.put_content_available(%Pigeon.APNS.Notification{})\n    %Pigeon.APNS.Notification{\n      collapse_id: nil,\n      device_token: nil,\n      expiration: nil,\n      priority: nil,\n      push_type: \"alert\",\n      id: nil,\n      payload: %{\"aps\" => %{\"content-available\" => 1}},\n      topic: nil\n    }","ref":"Pigeon.APNS.Notification.html#put_content_available/1-examples"},{"type":"function","title":"Pigeon.APNS.Notification.put_custom/2","doc":"Puts custom data in push payload.","ref":"Pigeon.APNS.Notification.html#put_custom/2"},{"type":"function","title":"Examples - Pigeon.APNS.Notification.put_custom/2","doc":"iex> n = Pigeon.APNS.Notification.new(\"test message\", \"device token\")\n    iex> Pigeon.APNS.Notification.put_custom(n, %{\"custom-key\" => 1234})\n    %Pigeon.APNS.Notification{\n      collapse_id: nil,\n      device_token: \"device token\",\n      expiration: nil,\n      priority: nil,\n      push_type: \"alert\",\n      id: nil,\n      payload: %{\"aps\" => %{\"alert\" => \"test message\"}, \"custom-key\" => 1234},\n      topic: nil\n    }","ref":"Pigeon.APNS.Notification.html#put_custom/2-examples"},{"type":"function","title":"Pigeon.APNS.Notification.put_interruption_level/2","doc":"Updates `\"interruption-level\"` key in push payload.\n\nUsed for managing time sensitive notifications","ref":"Pigeon.APNS.Notification.html#put_interruption_level/2"},{"type":"function","title":"Examples - Pigeon.APNS.Notification.put_interruption_level/2","doc":"iex> Pigeon.APNS.Notification.put_interruption_level(%Pigeon.APNS.Notification{}, \"time-sensitive\")\n    %Pigeon.APNS.Notification{\n      collapse_id: nil,\n      device_token: nil,\n      expiration: nil,\n      priority: nil,\n      push_type: \"alert\",\n      id: nil,\n      payload: %{\"aps\" => %{\"interruption-level\" => \"time-sensitive\"}},\n      topic: nil\n    }","ref":"Pigeon.APNS.Notification.html#put_interruption_level/2-examples"},{"type":"function","title":"Pigeon.APNS.Notification.put_mutable_content/1","doc":"Sets `\"mutable-content\"` flag in push payload.\n\nUsed for notification service extensions (such as displaying custom media).","ref":"Pigeon.APNS.Notification.html#put_mutable_content/1"},{"type":"function","title":"Examples - Pigeon.APNS.Notification.put_mutable_content/1","doc":"iex> Pigeon.APNS.Notification.put_mutable_content(%Pigeon.APNS.Notification{})\n    %Pigeon.APNS.Notification{\n      collapse_id: nil,\n      device_token: nil,\n      expiration: nil,\n      priority: nil,\n      push_type: \"alert\",\n      id: nil,\n      payload: %{\"aps\" => %{\"mutable-content\" => 1}},\n      topic: nil\n    }","ref":"Pigeon.APNS.Notification.html#put_mutable_content/1-examples"},{"type":"function","title":"Pigeon.APNS.Notification.put_sound/2","doc":"Updates `\"sound\"` key in push payload.\n\nUsed for custom notification sounds. Value should\nbe the name of the custom sound file in the application's binary.","ref":"Pigeon.APNS.Notification.html#put_sound/2"},{"type":"function","title":"Examples - Pigeon.APNS.Notification.put_sound/2","doc":"iex> Pigeon.APNS.Notification.put_sound(%Pigeon.APNS.Notification{}, \"custom.aiff\")\n    %Pigeon.APNS.Notification{\n      collapse_id: nil,\n      device_token: nil,\n      expiration: nil,\n      priority: nil,\n      push_type: \"alert\",\n      id: nil,\n      payload: %{\"aps\" => %{\"sound\" => \"custom.aiff\"}},\n      topic: nil\n    }","ref":"Pigeon.APNS.Notification.html#put_sound/2-examples"},{"type":"type","title":"Pigeon.APNS.Notification.response/0","doc":"APNS push response\n\n- nil - Push has not been sent yet.\n- `:success` - Push was successfully sent.\n- `t:Pigeon.APNS.Error.error_response/0` - Push attempted but\n   server responded with error.\n- `:timeout` - Internal error. Push did not reach APNS servers.","ref":"Pigeon.APNS.Notification.html#t:response/0"},{"type":"type","title":"Pigeon.APNS.Notification.t/0","doc":"APNS notification","ref":"Pigeon.APNS.Notification.html#t:t/0"},{"type":"type","title":"Examples - Pigeon.APNS.Notification.t/0","doc":"%Pigeon.APNS.Notification{\n        collapse_id: nil,\n        device_token: \"device token\",\n        expiration: nil,\n        priority: nil,\n        push_type: \"alert\",\n        id: nil, # Set on push response if nil\n        payload: %{\"aps\" => %{\"alert\" => \"push message\"}},\n        response: nil, # Set on push response\n        topic: \"com.example.YourApp\"\n    }","ref":"Pigeon.APNS.Notification.html#t:t/0-examples"},{"type":"protocol","title":"Pigeon.Configurable","doc":"","ref":"Pigeon.Configurable.html"},{"type":"function","title":"Pigeon.Configurable.close/1","doc":"","ref":"Pigeon.Configurable.html#close/1"},{"type":"function","title":"Pigeon.Configurable.connect/1","doc":"","ref":"Pigeon.Configurable.html#connect/1"},{"type":"function","title":"Pigeon.Configurable.handle_end_stream/4","doc":"","ref":"Pigeon.Configurable.html#handle_end_stream/4"},{"type":"function","title":"Pigeon.Configurable.max_demand/1","doc":"","ref":"Pigeon.Configurable.html#max_demand/1"},{"type":"function","title":"Pigeon.Configurable.push_headers/3","doc":"","ref":"Pigeon.Configurable.html#push_headers/3"},{"type":"function","title":"Pigeon.Configurable.push_payload/3","doc":"","ref":"Pigeon.Configurable.html#push_payload/3"},{"type":"function","title":"Pigeon.Configurable.schedule_ping/1","doc":"Schedules connection ping if necessary.","ref":"Pigeon.Configurable.html#schedule_ping/1"},{"type":"function","title":"Examples - Pigeon.Configurable.schedule_ping/1","doc":"iex> schedule_ping(%Pigeon.APNS.Config{ping_period: 2})\n    iex> receive do\n    ...>   :ping -> \"Got ping!\"\n    ...> after\n    ...>   5000 -> \"No ping received.\"\n    ...> end\n    \"Got ping!\"\n\n    iex> schedule_ping(%Pigeon.FCM.Config{})\n    iex> receive do\n    ...>   :ping -> \"Got ping!\"\n    ...> after\n    ...>   5000 -> \"No ping received.\"\n    ...> end\n    \"No ping received.\"","ref":"Pigeon.Configurable.html#schedule_ping/1-examples"},{"type":"function","title":"Pigeon.Configurable.validate!/1","doc":"","ref":"Pigeon.Configurable.html#validate!/1"},{"type":"function","title":"Pigeon.Configurable.worker_name/1","doc":"Returns worker name for config.","ref":"Pigeon.Configurable.html#worker_name/1"},{"type":"function","title":"Examples - Pigeon.Configurable.worker_name/1","doc":"iex> worker_name(%Pigeon.APNS.Config{name: :test})\n    :test\n\n    iex> worker_name(%Pigeon.FCM.Config{name: :another})\n    :another","ref":"Pigeon.Configurable.html#worker_name/1-examples"},{"type":"type","title":"Pigeon.Configurable.sock/0","doc":"","ref":"Pigeon.Configurable.html#t:sock/0"},{"type":"type","title":"Pigeon.Configurable.t/0","doc":"All the types that implement this protocol.","ref":"Pigeon.Configurable.html#t:t/0"},{"type":"protocol","title":"Pigeon.Encodable","doc":"","ref":"Pigeon.Encodable.html"},{"type":"function","title":"Pigeon.Encodable.binary_payload/1","doc":"","ref":"Pigeon.Encodable.html#binary_payload/1"},{"type":"type","title":"Pigeon.Encodable.t/0","doc":"All the types that implement this protocol.","ref":"Pigeon.Encodable.html#t:t/0"},{"type":"module","title":"Pigeon.FCM","doc":"Firebase Cloud Messaging (FCM)","ref":"Pigeon.FCM.html"},{"type":"function","title":"Pigeon.FCM.push/2","doc":"Sends a push over FCM.","ref":"Pigeon.FCM.html#push/2"},{"type":"function","title":"Examples - Pigeon.FCM.push/2","doc":"iex> n = Pigeon.FCM.Notification.new(\"regId\", %{}, %{\"message\" => \"123\"})\n    iex> Pigeon.FCM.push(n)\n    %Pigeon.FCM.Notification{message_id: nil,\n     payload: %{\"data\" => %{\"message\" => \"123\"}}, priority: :normal,\n     registration_id: \"regId\", status: :success, response:\n     [invalid_registration: \"regId\"]}\n\n    iex> n = Pigeon.FCM.Notification.new(\"regId\", %{}, %{\"message\" => \"123\"})\n    iex> Pigeon.FCM.push(n, on_response: nil)\n    :ok\n\n    iex> n = Pigeon.FCM.Notification.new([\"regId\", \"regId\"], %{},\n    ...> %{\"message\" => \"123\"})\n    iex> Pigeon.FCM.push(n)\n    %Pigeon.FCM.Notification{message_id: nil,\n     payload: %{\"data\" => %{\"message\" => \"123\"}}, priority: :normal,\n     registration_id: [\"regId\", \"regId\"], status: :success,\n     response: [invalid_registration: \"regId\",\n     invalid_registration: \"regId\"]}\n\n    iex> n = Pigeon.FCM.Notification.new([\"regId\", \"regId\"], %{},\n    ...> %{\"message\" => \"test\"})\n    iex> notifs = Pigeon.FCM.push([n, n])\n    iex> Enum.map(notifs, & &1.response)\n    [[invalid_registration: \"regId\", invalid_registration: \"regId\"],\n     [invalid_registration: \"regId\", invalid_registration: \"regId\"]]","ref":"Pigeon.FCM.html#push/2-examples"},{"type":"function","title":"Pigeon.FCM.start_connection/1","doc":"Starts FCM worker connection with given config or name.","ref":"Pigeon.FCM.html#start_connection/1"},{"type":"function","title":"Examples - Pigeon.FCM.start_connection/1","doc":"iex> config = Pigeon.FCM.Config.new(:fcm_default)\n    iex> {:ok, pid} = Pigeon.FCM.start_connection(%{config | name: nil})\n    iex> Process.alive?(pid)\n    true","ref":"Pigeon.FCM.html#start_connection/1-examples"},{"type":"function","title":"Pigeon.FCM.stop_connection/1","doc":"Stops existing FCM worker connection.","ref":"Pigeon.FCM.html#stop_connection/1"},{"type":"function","title":"Examples - Pigeon.FCM.stop_connection/1","doc":"iex> config = Pigeon.FCM.Config.new(:fcm_default)\n    iex> {:ok, pid} = Pigeon.FCM.start_connection(%{config | name: nil})\n    iex> Pigeon.FCM.stop_connection(pid)\n    :ok\n    iex> :timer.sleep(500)\n    iex> Process.alive?(pid)\n    false","ref":"Pigeon.FCM.html#stop_connection/1-examples"},{"type":"type","title":"Pigeon.FCM.notification/0","doc":"Can be either a single notification or a list.","ref":"Pigeon.FCM.html#t:notification/0"},{"type":"type","title":"Pigeon.FCM.on_response/0","doc":"Async callback for push notification response.","ref":"Pigeon.FCM.html#t:on_response/0"},{"type":"type","title":"Examples - Pigeon.FCM.on_response/0","doc":"handler = fn(n) ->\n      case n.status do\n        :success ->\n          bad_regids = FCM.Notification.remove?(n)\n          to_retry = FCM.Notification.retry?(n)\n          # Handle updated regids, remove bad ones, etc\n        :unauthorized ->\n          # Bad FCM key\n        error ->\n          # Some other error\n      end\n    end\n\n    n = Pigeon.FCM.Notification.new(\"device token\", %{}, %{\"message\" => \"test\"})\n    Pigeon.FCM.push(n, on_response: handler)","ref":"Pigeon.FCM.html#t:on_response/0-examples"},{"type":"type","title":"Pigeon.FCM.push_opts/0","doc":"Options for sending push notifications.\n\n- `:to` - Defines worker to process push. Defaults to `:fcm_default`\n- `:on_response` - Optional async callback triggered on receipt of push.\n  See `t:on_response/0`\n- `:timeout` - Specifies timeout for push responses. Useful if sending large\n  batches synchronously.","ref":"Pigeon.FCM.html#t:push_opts/0"},{"type":"module","title":"Pigeon.FCM.Config","doc":"FCM Configuration for Pigeon","ref":"Pigeon.FCM.Config.html"},{"type":"function","title":"Pigeon.FCM.Config.new/1","doc":"Returns a new `FCM.Config` with given `opts`.","ref":"Pigeon.FCM.Config.html#new/1"},{"type":"function","title":"Examples - Pigeon.FCM.Config.new/1","doc":"iex> Pigeon.FCM.Config.new(\n    ...>   name: :test,\n    ...>   key: \"fcm_key\",\n    ...>   uri: 'test.server.example.com',\n    ...>   port: 5228\n    ...> )\n    %Pigeon.FCM.Config{key: \"fcm_key\", name: :test,\n    port: 5228, uri: 'test.server.example.com'}","ref":"Pigeon.FCM.Config.html#new/1-examples"},{"type":"type","title":"Pigeon.FCM.Config.t/0","doc":"","ref":"Pigeon.FCM.Config.html#t:t/0"},{"type":"module","title":"Pigeon.FCM.Notification","doc":"Defines FCM notification struct and convenience constructor functions.","ref":"Pigeon.FCM.Notification.html"},{"type":"function","title":"Pigeon.FCM.Notification.new/3","doc":"Creates `FCM.Notification` struct with device registration IDs and optional\nnotification and data payloads.","ref":"Pigeon.FCM.Notification.html#new/3"},{"type":"function","title":"Examples - Pigeon.FCM.Notification.new/3","doc":"iex> Pigeon.FCM.Notification.new(\"reg ID\")\n    %Pigeon.FCM.Notification{\n      payload: %{},\n      registration_id: \"reg ID\",\n      priority: :normal\n    }\n\n    iex> Pigeon.FCM.Notification.new(\"reg ID\", %{\"body\" => \"test message\"})\n    %Pigeon.FCM.Notification{\n      payload: %{\"notification\" => %{\"body\" => \"test message\"}},\n      registration_id: \"reg ID\",\n      priority: :normal\n    }\n\n    iex> Pigeon.FCM.Notification.new(\"reg ID\", %{\"body\" => \"test message\"},\n    ...> %{\"key\" => \"value\"})\n    %Pigeon.FCM.Notification{\n      payload: %{\n        \"data\" => %{\"key\" => \"value\"},\n        \"notification\" => %{\"body\" => \"test message\"}\n      },\n      registration_id: \"reg ID\",\n      priority: :normal\n    }\n\n    iex> regids = Enum.map(0..1_499, fn(_x) -> \"reg ID\" end)\n    iex> [n1 | [n2]] = Pigeon.FCM.Notification.new(regids,\n    ...> %{\"body\" => \"test message\"}, %{\"key\" => \"value\"})\n    iex> Enum.count(n1.registration_id)\n    1000\n    iex> Enum.count(n2.registration_id)\n    500","ref":"Pigeon.FCM.Notification.html#new/3-examples"},{"type":"function","title":"Pigeon.FCM.Notification.put_collapse_key/2","doc":"Updates `\"collapse_key\"` key.","ref":"Pigeon.FCM.Notification.html#put_collapse_key/2"},{"type":"function","title":"Examples - Pigeon.FCM.Notification.put_collapse_key/2","doc":"iex> put_collapse_key(%Pigeon.FCM.Notification{}, \"Updates available\")\n    %Pigeon.FCM.Notification{collapse_key: \"Updates available\"}","ref":"Pigeon.FCM.Notification.html#put_collapse_key/2-examples"},{"type":"function","title":"Pigeon.FCM.Notification.put_condition/2","doc":"Updates `\"condition\"` key.\n\nSupported condition: Topic, formatted as \"'yourTopic' in topics\". This value\nis case-insensitive.\n\nSupported operators: &&, ||. Maximum two operators per topic message supported.","ref":"Pigeon.FCM.Notification.html#put_condition/2"},{"type":"function","title":"Examples - Pigeon.FCM.Notification.put_condition/2","doc":"iex> put_condition(%Pigeon.FCM.Notification{}, \"'test' in topics\")\n    %Pigeon.FCM.Notification{condition: \"'test' in topics\"}","ref":"Pigeon.FCM.Notification.html#put_condition/2-examples"},{"type":"function","title":"Pigeon.FCM.Notification.put_content_available/2","doc":"Updates `\"content_available\"` key.\n\nOn iOS, use this field to represent content-available in the APNs payload.\nWhen a notification or message is sent and this is set to true, an inactive\nclient app is awoken, and the message is sent through APNs as a silent\nnotification and not through the FCM connection server. Note that silent\nnotifications in APNs are not guaranteed to be delivered, and can depend on\nfactors such as the user turning on Low Power Mode, force quitting the app,\netc. On Android, data messages wake the app by default. On Chrome, currently\nnot supported.","ref":"Pigeon.FCM.Notification.html#put_content_available/2"},{"type":"function","title":"Examples - Pigeon.FCM.Notification.put_content_available/2","doc":"iex> put_content_available(%Pigeon.FCM.Notification{}, true)\n    %Pigeon.FCM.Notification{content_available: true}","ref":"Pigeon.FCM.Notification.html#put_content_available/2-examples"},{"type":"function","title":"Pigeon.FCM.Notification.put_data/2","doc":"Updates `\"data\"` key in push payload.\n\nThis parameter specifies the custom key-value pairs of the message's payload.\n\nFor example, with data:{\"score\":\"3x1\"}:\n\nOn iOS, if the message is sent via APNs, it represents the custom data fields.\nIf it is sent via FCM connection server, it would be represented as key value\ndictionary in AppDelegate application:didReceiveRemoteNotification:.\n\nOn Android, this would result in an intent extra named score with the string\nvalue 3x1.\n\nThe key should not be a reserved word (\"from\" or any word starting with\n\"google\" or \"gcm\"). Do not use any of the words defined in this table\n(such as collapse_key).\n\nValues in string types are recommended. You have to convert values in objects\nor other non-string data types (e.g., integers or booleans) to string.","ref":"Pigeon.FCM.Notification.html#put_data/2"},{"type":"function","title":"Examples - Pigeon.FCM.Notification.put_data/2","doc":"iex> put_data(%Pigeon.FCM.Notification{}, %{\"key\" => 1234})\n    %Pigeon.FCM.Notification{\n      payload: %{\"data\" => %{\"key\" => 1234}},\n      registration_id: nil\n    }","ref":"Pigeon.FCM.Notification.html#put_data/2-examples"},{"type":"function","title":"Pigeon.FCM.Notification.put_dry_run/1","doc":"Sets `\"dry_run\"` key to true.\n\nThis parameter, when set to true, allows developers to test a request without\nactually sending a message.\n\nThe default value is false.","ref":"Pigeon.FCM.Notification.html#put_dry_run/1"},{"type":"function","title":"Examples - Pigeon.FCM.Notification.put_dry_run/1","doc":"iex> put_dry_run(%Pigeon.FCM.Notification{})\n    %Pigeon.FCM.Notification{dry_run: true}","ref":"Pigeon.FCM.Notification.html#put_dry_run/1-examples"},{"type":"function","title":"Pigeon.FCM.Notification.put_mutable_content/2","doc":"Updates `\"mutable_content\"` key.\n\nCurrently for iOS 10+ devices only. On iOS, use this field to represent\nmutable-content in the APNs payload. When a notification is sent and this is\nset to true, the content of the notification can be modified before it is\ndisplayed, using a Notification Service app extension. This parameter will be\nignored for Android and web.","ref":"Pigeon.FCM.Notification.html#put_mutable_content/2"},{"type":"function","title":"Examples - Pigeon.FCM.Notification.put_mutable_content/2","doc":"iex> put_mutable_content(%Pigeon.FCM.Notification{}, true)\n    %Pigeon.FCM.Notification{mutable_content: true}","ref":"Pigeon.FCM.Notification.html#put_mutable_content/2-examples"},{"type":"function","title":"Pigeon.FCM.Notification.put_notification/2","doc":"Updates `\"notification\"` key in push payload.\n\nThis parameter specifies the predefined, user-visible key-value pairs of the\nnotification payload. See Notification payload support for detail. For more\ninformation about notification message and data message options, see\n[Message types](https://firebase.google.com/docs/cloud-messaging/concept-options#notifications_and_data_messages).\nIf a notification payload is provided, or the content_available option is set\nto true for a message to an iOS device, the message is sent through APNs,\notherwise it is sent through the FCM connection server.","ref":"Pigeon.FCM.Notification.html#put_notification/2"},{"type":"function","title":"Examples - Pigeon.FCM.Notification.put_notification/2","doc":"iex> put_notification(%Pigeon.FCM.Notification{},\n    ...> %{\"body\" => \"message\"})\n    %Pigeon.FCM.Notification{\n      payload: %{\"notification\" => %{\"body\" => \"message\"}},\n      registration_id: nil\n    }","ref":"Pigeon.FCM.Notification.html#put_notification/2-examples"},{"type":"function","title":"Pigeon.FCM.Notification.put_priority/2","doc":"Updates `\"priority\"` key.\n\nSets the priority of the message. Valid values are \"normal\" and \"high.\" On\niOS, these correspond to APNs priorities 5 and 10.\n\nBy default, notification messages are sent with high priority, and data\nmessages are sent with normal priority. Normal priority optimizes the client\napp's battery consumption and should be used unless immediate delivery is\nrequired. For messages with normal priority, the app may receive the message\nwith unspecified delay.\n\nWhen a message is sent with high priority, it is sent immediately, and the app\ncan display a notification.","ref":"Pigeon.FCM.Notification.html#put_priority/2"},{"type":"function","title":"Examples - Pigeon.FCM.Notification.put_priority/2","doc":"iex> put_priority(%Pigeon.FCM.Notification{}, :normal)\n    %Pigeon.FCM.Notification{priority: :normal}\n\n    iex> put_priority(%Pigeon.FCM.Notification{}, :high)\n    %Pigeon.FCM.Notification{priority: :high}\n\n    iex> put_priority(%Pigeon.FCM.Notification{priority: :normal}, :bad)\n    %Pigeon.FCM.Notification{priority: :normal}","ref":"Pigeon.FCM.Notification.html#put_priority/2-examples"},{"type":"function","title":"Pigeon.FCM.Notification.put_restricted_package_name/2","doc":"Updates `\"restricted_package_name\"` key.\n\nThis parameter specifies the package name of the application where the\nregistration tokens must match in order to receive the message. (Only affects android)","ref":"Pigeon.FCM.Notification.html#put_restricted_package_name/2"},{"type":"function","title":"Examples - Pigeon.FCM.Notification.put_restricted_package_name/2","doc":"iex> put_restricted_package_name(%Pigeon.FCM.Notification{}, \"com.example.app\")\n    %Pigeon.FCM.Notification{restricted_package_name: \"com.example.app\"}","ref":"Pigeon.FCM.Notification.html#put_restricted_package_name/2-examples"},{"type":"function","title":"Pigeon.FCM.Notification.put_time_to_live/2","doc":"Updates `\"time_to_live\"` key.\n\nThis parameter specifies how long (in seconds) the message should be kept in\nFCM storage if the device is offline. The maximum time to live supported is 4\nweeks, and the default value is 4 weeks. For more information, see\n[Setting the lifespan of a message](https://firebase.google.com/docs/cloud-messaging/concept-options#ttl).","ref":"Pigeon.FCM.Notification.html#put_time_to_live/2"},{"type":"function","title":"Examples - Pigeon.FCM.Notification.put_time_to_live/2","doc":"iex> put_time_to_live(%Pigeon.FCM.Notification{}, 60 * 60 * 24)\n    %Pigeon.FCM.Notification{time_to_live: 86_400}","ref":"Pigeon.FCM.Notification.html#put_time_to_live/2-examples"},{"type":"function","title":"Pigeon.FCM.Notification.remove?/1","doc":"Returns a list of registration IDs that should be removed","ref":"Pigeon.FCM.Notification.html#remove?/1"},{"type":"function","title":"Examples - Pigeon.FCM.Notification.remove?/1","doc":"iex> n = %Pigeon.FCM.Notification{response: [\n    ...> {:success, \"regid1\"}, {:invalid_registration, \"regid2\"},\n    ...> {:success, \"regid3\"}, {:update, {\"regid4\", \"new_regid4\"}},\n    ...> {:not_registered, \"regid5\"}, {:unavailable, \"regid6\"}]}\n    iex> remove?(n)\n    [\"regid2\", \"regid5\"]","ref":"Pigeon.FCM.Notification.html#remove?/1-examples"},{"type":"function","title":"Pigeon.FCM.Notification.retry?/1","doc":"Returns a list of registration IDs that should be retried","ref":"Pigeon.FCM.Notification.html#retry?/1"},{"type":"function","title":"Examples - Pigeon.FCM.Notification.retry?/1","doc":"iex> n = %Pigeon.FCM.Notification{response: [\n    ...> {:success, \"regid1\"}, {:invalid_registration, \"regid2\"},\n    ...> {:success, \"regid3\"}, {:update, {\"regid4\", \"new_regid4\"}},\n    ...> {:not_registered, \"regid5\"}, {:unavailable, \"regid6\"}]}\n    iex> retry?(n)\n    [\"regid6\"]","ref":"Pigeon.FCM.Notification.html#retry?/1-examples"},{"type":"function","title":"Pigeon.FCM.Notification.success?/1","doc":"Returns a list of successful registration IDs","ref":"Pigeon.FCM.Notification.html#success?/1"},{"type":"function","title":"Examples - Pigeon.FCM.Notification.success?/1","doc":"iex> n = %Pigeon.FCM.Notification{response: [\n    ...> {:success, \"regid1\"}, {:invalid_registration, \"regid2\"},\n    ...> {:success, \"regid3\"}, {:update, {\"regid4\", \"new_regid4\"}},\n    ...> {:not_registered, \"regid5\"}, {:unavailable, \"regid6\"}]}\n    iex> success?(n)\n    [\"regid1\", \"regid3\"]","ref":"Pigeon.FCM.Notification.html#success?/1-examples"},{"type":"function","title":"Pigeon.FCM.Notification.update?/1","doc":"Returns a list of registration IDs and their corresponding new ID","ref":"Pigeon.FCM.Notification.html#update?/1"},{"type":"function","title":"Examples - Pigeon.FCM.Notification.update?/1","doc":"iex> n = %Pigeon.FCM.Notification{response: [\n    ...> {:success, \"regid1\"}, {:invalid_registration, \"regid2\"},\n    ...> {:success, \"regid3\"}, {:update, {\"regid4\", \"new_regid4\"}},\n    ...> {:not_registered, \"regid5\"}, {:unavailable, \"regid6\"}]}\n    iex> update?(n)\n    [{\"regid4\", \"new_regid4\"}]","ref":"Pigeon.FCM.Notification.html#update?/1-examples"},{"type":"type","title":"Pigeon.FCM.Notification.regid_error_response/0","doc":"","ref":"Pigeon.FCM.Notification.html#t:regid_error_response/0"},{"type":"type","title":"Pigeon.FCM.Notification.regid_response/0","doc":"FCM push response for individual registration IDs\n\n- `{:success, \"reg_id\"}` - Push was successfully sent\n- `{:update, {\"reg_id\", \"new_reg_id\"}}` - Push successful but user should\n  use new registration ID for future pushes\n- `{regid_error_response, \"reg_id\"}` - Push attempted but server responded\n  with error","ref":"Pigeon.FCM.Notification.html#t:regid_response/0"},{"type":"type","title":"Pigeon.FCM.Notification.status/0","doc":"Status of FCM request\n\n- `:success` - Notification was processed successfully\n- `:timeout` - Worker did not respond within timeout. This is likely an\n   internal error\n- `:unauthorized` - Bad FCM key\n- `:malformed_json` - Push payload was invalid JSON\n- `:internal_server_error` - FCM server encountered an error while trying\n   to process the request\n- `:unavailable` - FCM server couldn't process the request in time","ref":"Pigeon.FCM.Notification.html#t:status/0"},{"type":"type","title":"Pigeon.FCM.Notification.t/0","doc":"","ref":"Pigeon.FCM.Notification.html#t:t/0"},{"type":"behaviour","title":"Pigeon.Http2.Client","doc":"HTTP2 client adapter behaviour\n\nUses `Kadabra` by default, but any client can be used\nwith the appropriate adapter.","ref":"Pigeon.Http2.Client.html"},{"type":"behaviour","title":"Writing a Custom Client Adapter - Pigeon.Http2.Client","doc":"`Pigeon.Worker` relies on all of the callbacks for important\nconnection functionality.\n\n* `start/0`\n  * Starts the client application when `Pigeon` starts.\n* `connect/3`\n  * Opens a socket connection. Must return `{:ok, pid}` or\n    `{:error, reason}`. If an error, `Pigeon.Worker` will retry\n    the callback two more times.\n* `send_ping/1`\n  * Sends an HTTP2 ping. `Pigeon.Worker` periodically sends pings to keep\n    the connection alive. Client adapters must support pings for\n    APNS-configured workers, though it is not necessary if only using FCM.\n* `send_request/3`\n  * Makes an HTTP2 request. `Pigeon.Worker` does not handle synchronous\n    requests and will ignore the result of this callback. If the client\n    is synchronous, the adapter will need to explicitly send a message\n    back to the `Pigeon.Worker`.\n* `handle_end_stream/2`\n  * All incoming messages on the worker are passed through this callback.\n    Must return `{:ok, %Pigeon.Http2.Stream{...}}` if it is a valid\n    `END_STREAM` response for the adapter. All other messages are ignored.\n\n**Example implementation for Kadabra**\n\n    if Code.ensure_loaded?(Kadabra) do\n    defmodule Pigeon.Http2.Client.Kadabra do\n      @moduledoc false\n\n      @behaviour Pigeon.Http2.Client\n\n      def start do\n        Application.ensure_all_started(:kadabra)\n      end\n\n      def connect(uri, scheme, opts) do\n        Kadabra.open(uri, scheme, opts)\n      end\n\n      def send_request(pid, headers, data) do\n        Kadabra.request(pid, headers, data)\n      end\n\n      def send_ping(pid) do\n        Kadabra.ping(pid)\n      end\n\n      def handle_end_stream({:end_stream, %{id: id,\n                                            status: status,\n                                            headers: headers,\n                                            body: body}}, _state) do\n        {:ok, %Pigeon.Http2.Stream{\n            id: id,\n            status: status,\n            headers: headers,\n            body: body\n        }}\n      end\n      def handle_end_stream(msg, _state), do: msg\n    end\n    end","ref":"Pigeon.Http2.Client.html#module-writing-a-custom-client-adapter"},{"type":"behaviour","title":"Using Your Client Adapter - Pigeon.Http2.Client","doc":"Once implemented, specify it in your config. Pigeon will use it for all\nHTTP2 connections.\n\n    config :pigeon, http2_client: Pigeon.YourCustomAdapter","ref":"Pigeon.Http2.Client.html#module-using-your-client-adapter"},{"type":"callback","title":"Pigeon.Http2.Client.connect/3","doc":"","ref":"Pigeon.Http2.Client.html#c:connect/3"},{"type":"function","title":"Pigeon.Http2.Client.default/0","doc":"Default http2 client to use.\n\nWhen not configured, defaults to `Pigeon.Http2.Client.Kadabra`","ref":"Pigeon.Http2.Client.html#default/0"},{"type":"function","title":"Examples - Pigeon.Http2.Client.default/0","doc":"iex> Pigeon.Http2.Client.default\n    Pigeon.Http2.Client.Kadabra","ref":"Pigeon.Http2.Client.html#default/0-examples"},{"type":"callback","title":"Pigeon.Http2.Client.handle_end_stream/2","doc":"","ref":"Pigeon.Http2.Client.html#c:handle_end_stream/2"},{"type":"callback","title":"Pigeon.Http2.Client.send_ping/1","doc":"","ref":"Pigeon.Http2.Client.html#c:send_ping/1"},{"type":"callback","title":"Pigeon.Http2.Client.send_request/3","doc":"","ref":"Pigeon.Http2.Client.html#c:send_request/3"},{"type":"callback","title":"Pigeon.Http2.Client.start/0","doc":"","ref":"Pigeon.Http2.Client.html#c:start/0"},{"type":"type","title":"Pigeon.Http2.Client.uri/0","doc":"","ref":"Pigeon.Http2.Client.html#t:uri/0"},{"type":"exception","title":"Pigeon.ConfigError","doc":"","ref":"Pigeon.ConfigError.html"},{"type":"extras","title":"Getting Started","doc":"![logo](https://raw.githubusercontent.com/codedge-llc/pigeon/master/docs/logo.png)\n\n> HTTP2-compliant wrapper for sending iOS and Android push notifications.\n\n[![Build Status](https://travis-ci.org/codedge-llc/pigeon.svg?branch=master)](https://travis-ci.org/codedge-llc/pigeon)\n[![Coverage Status](https://coveralls.io/repos/github/codedge-llc/pigeon/badge.svg)](https://coveralls.io/github/codedge-llc/pigeon)\n[![Hex.pm](http://img.shields.io/hexpm/v/pigeon.svg)](https://hex.pm/packages/pigeon)\n[![Hex.pm](http://img.shields.io/hexpm/dt/pigeon.svg)](https://hex.pm/packages/pigeon)","ref":"getting-started.html"},{"type":"extras","title":"Installation - Getting Started","doc":"Add pigeon and kadabra as `mix.exs` dependencies:\n\n```elixir\ndef deps do\n  [\n    {:pigeon, \"~> 1.6.2\"},\n    {:kadabra, \"~> 0.6.0\"}\n  ]\nend\n```","ref":"getting-started.html#installation"},{"type":"extras","title":"Quickstart Guides - Getting Started","doc":"#","ref":"getting-started.html#quickstart-guides"},{"type":"extras","title":"Apple iOS (APNS) - Getting Started","doc":"1. Add a default worker config to your mix config. See [the detailed\n   docs](https://hexdocs.pm/pigeon/apns-apple-ios.html) for setting up your\n   certificate and key.\n\n   ```elixir\n   config :pigeon, :apns,\n     apns_default: %{\n       cert: \"cert.pem\",\n       key: \"key_unencrypted.pem\",\n       mode: :dev\n     }\n   ```\n\n   This config sets up a default connection to APNS servers. `cert` and\n   `key` can be any of the following:\n\n   - Static file path\n   - Full-text string of the file contents\n   - `{:my_app, \"certs/cert.pem\"}` (indicates path relative to the `priv`\n     folder of the given application)\n\n   Alternatively, you can use token based authentication:\n\n   ```elixir\n   config :pigeon, :apns,\n     apns_default: %{\n       key: \"AuthKey.p8\",\n       key_identifier: \"ABC1234567\",\n       team_id: \"DEF8901234\",\n       mode: :dev\n     }\n   ```\n\n   - `:key` - Created and downloaded via your developer account. Like `:cert`\n     this can be a file path, file contents string or tuple\n   - `:key_identifier` - The 10-character key identifier associated with\n     `:key`, obtained from your developer account\n   - `:team_id` - Your 10-character Team ID, obtained from your developer\n     account\n\n2. Create a notification packet. **Note: Your push topic is generally the\n   app's bundle identifier.**\n\n   ```elixir\n   iex> n = Pigeon.APNS.Notification.new(\"your message\", \"your device token\", \"your push topic (optional)\")\n   ```\n\n3. Send the packet. Pushes are synchronous and return the notification with an\n   updated `:response` key.\n\n   ```elixir\n   iex> Pigeon.APNS.push(n)\n   %Pigeon.APNS.Notification{device_token: \"your device token\",\n    expiration: nil, id: \"963B9FDA-EA60-E869-AAB5-9C88C8E7396B\",\n    payload: %{\"aps\" => %{\"alert\" => \"your message\"}}, response: :success,\n    topic: \"your push topic\"}\n\n   # Add an `:on_response` callback for async pushes.\n   iex> Pigeon.APNS.push(n, on_response: fn(x) -> IO.inspect(x) end)\n   :ok\n   ```\n\nAdditional documentation: [APNS (Apple iOS)](https://hexdocs.pm/pigeon/apns-apple-ios.html)\n\n#","ref":"getting-started.html#apple-ios-apns"},{"type":"extras","title":"Android (FCM) - Getting Started","doc":"Looking for GCM? Try `v0.13` or earlier.\n\n1. Add a default worker config to your mix config.\n\n   ```elixir\n   config :pigeon, :fcm,\n     fcm_default: %{\n       key: \"your_fcm_key_here\"\n     }\n   ```\n\n2. Create a notification packet. FCM notifications support\n\n   ```elixir\n   iex> msg = %{ \"body\" => \"your message\" }\n   iex> n = Pigeon.FCM.Notification.new(\"your device registration ID\", msg)\n   ```\n\n3. Send the packet. Pushes are synchronous and return the notification with\n   updated `:status` and `:response` keys. If `:status` is success, `:response`\n   will contain a keyword list of individual registration ID responses.\n\n   ```elixir\n   iex> Pigeon.FCM.push(n)\n   %Pigeon.FCM.Notification{message_id: \"0:1512580747839227%8911a9178911a917\",\n    payload: %{\"notification\" => %{\"body\" => \"your message\"}}, priority: :normal,\n    registration_id: \"your device registration ID\",\n    response: [success: \"your device registration ID\"],\n    status: :success}\n\n   # Add an `:on_response` callback for async pushes.\n   iex> Pigeon.FCM.push(n, on_response: fn(x) -> IO.inspect(x) end)\n   :ok\n   ```\n\nAdditional documentation: [FCM (Android)](https://hexdocs.pm/pigeon/fcm-android.html)\n\n#","ref":"getting-started.html#android-fcm"},{"type":"extras","title":"Amazon Android (ADM) - Getting Started","doc":"1. Add a default worker config to your mix config.\n\n   ```elixir\n   config :pigeon, :adm,\n     adm_default: %{\n       client_id: \"your_oauth2_client_id_here\",\n       client_secret: \"your_oauth2_client_secret_here\"\n     }\n   ```\n\n2. Create a notification packet.\n\n   ```elixir\n   iex> msg = %{ \"body\" => \"your message\" }\n   iex> n = Pigeon.ADM.Notification.new(\"your device registration ID\", msg)\n   ```\n\n3. Send the packet.\n\n   ```elixir\n   iex> Pigeon.ADM.push(n)\n   %Pigeon.ADM.Notification{consolidation_key: nil,\n    expires_after: 604800, md5: \"M13RuG4uDWqajseQcCiyiw==\",\n    payload: %{\"data\" => %{\"body\" => \"your message\"}},\n    registration_id: \"your device registration ID\",\n    response: :success, updated_registration_id: nil}\n\n   # Add an `:on_response` callback for async pushes.\n   iex> Pigeon.ADM.push(n, on_response: fn(x) -> IO.inspect(x) end)\n   :ok\n   ```\n\nAdditional documentation: [ADM (Amazon Android)](https://hexdocs.pm/pigeon/adm-amazon-android.html)","ref":"getting-started.html#amazon-android-adm"},{"type":"extras","title":"Startup Configuration of Push Workers - Getting Started","doc":"Workers may be specified at application startup by creating a module containing\nfunctions that return zero or more configuration structures appropriate to the\npush service being enabled.\n\nSpecify these in your `config.exs` as:\n\n```elixir\nconfig :pigeon, workers: [\n  {YourApp.Pigeon, :apns_config},\n  {YourApp.Pigeon, :fcm_config},\n  {YourApp.Pigeon, :adm_config}\n]\n```\n\nThese should be implemented as:\n\n```elixir\ndefmodule YourApp.Pigeon do\n  @moduledoc false\n\n  @push_mode if(Mix.env() == :production, do: :prod, else: :dev)\n\n  def apns_config do\n    Pigeon.APNS.ConfigParser.parse(\n      key: System.get_env(\"APNS_KEY\"),\n      key_identifier: System.get_env(\"APNS_KEY_ID\"),\n      team_id: System.get_env(\"APNS_TEAM_ID\"),\n      mode: @push_mode,\n      name: :apns_default\n    )\n  end\n\n  def fcm_config do\n    Pigeon.FCM.Config.new(name: :fcm_default, key: System.get_env(\"FCM_SERVER_KEY\"))\n  end\nend\n```\n\nIf your startup configuration requires reading your configuration from a\ndatabase or using another dependency that needs a database, startup is a little\nmore complex.\n\n1. Modify your `mix.exs` to _not_ start `pigeon` by default:\n\n   ```elixir\n   def deps do\n     [\n       {:pigeon, \"~> 1.3.1\", runtime: false},\n       {:kadabra, \"~> 0.4.4\"},\n       {:ecto, \"~> 2.0 or ~> 3.0\"}\n     ]\n   end\n   ```\n\n2. Modify `config.exs` to specify a single worker function:\n\n   ```elixir\n   config :pigeon, workers: [{YourApp.Pigeon, :config}]\n   ```\n\n3. Modify your main application to start `pigeon` after your `Repo` has been\n   started under your application’s supervision tree:\n\n   ```elixir\n   def start(_type, _args) do\n     children = [\n       YourApp.Repo,\n     ]\n\n     opts = [strategy: :one_for_one, name: YourApp.Supervisor]\n\n     with {:ok, sup} <- Supervisor.start_link(children, opts),\n         {:ok, _} <- Application.ensure_all_started(:pigeon, :permanent) do\n       {:ok, sup}\n     end\n   end\n   ```\n\n4. Implement your database query as part of your Pigeon module:\n\n   ```elixir\n   defmodule YourApp.Pigeon do\n     @moduledoc false\n\n     alias YourApp.{PushApplication, Repo}\n\n     def config_workers do\n       PushApplication\n       |> Repo.all()\n       |> Enum.map(&build_config/1)\n     end\n\n     defp build_config(%{type: \"apns\"} = config)\n       Pigeon.APNS.ConfigParser.parse(\n         key: config.key,\n         key_identifier: config.key_identifier,\n         team_id: config.team_id,\n         mode: config.mode,\n         name: Atom.to_string(config.name) # This is bad, but keep it simple!\n       )\n     end\n\n     defp build_config(%{type: \"fcm\"} = config) do\n       Pigeon.FCM.Config.new(\n         name: Atom.to_string(config.name),\n         key: config.key\n       )\n     end\n   end\n   ```","ref":"getting-started.html#startup-configuration-of-push-workers"},{"type":"extras","title":"APNS (Apple iOS)","doc":"# APNS (Apple iOS)","ref":"apns-apple-ios.html"},{"type":"extras","title":"Usage - APNS (Apple iOS)","doc":"1. Set your environment variables. See below for setting up your certificate and key.\n\n    ```elixir\n    config :pigeon, :apns,\n      apns_default: %{\n        cert: \"cert.pem\",\n        key: \"key_unencrypted.pem\",\n        mode: :dev\n      }\n    ```\n\n    This config sets up a `default` socket connection to send to APNS servers. `cert` and `key` can be any of the following:\n    * Static file path\n    * Full-text string of the file contents (useful for environment variables)\n    * `{:my_app, \"certs/cert.pem\"}` (indicates path relative to the `priv` folder of the given application)\n\n    Alternatively, you can use token based authentication:\n\n    ```elixir\n    config :pigeon, :apns,\n      apns_default: %{\n        key: \"AuthKey.p8\",\n        key_identifier: \"ABC1234567\",\n        team_id: \"DEF8901234\",\n        mode: :dev\n      }\n    ```\n\n    * `:key` - Created and downloaded via your developer account. Like `:cert` this can be a file path, file contents string or tuple\n    * `:key_identifier` - The 10-character key identifier associated with `:key`, obtained from your developer account\n    * `:team_id` - Your 10-character Team ID, obtained from your developer account\n\n2. Create a notification packet. **Note: Your push topic is generally the app's bundle identifier.**\n\n    ```elixir\n    n = Pigeon.APNS.Notification.new(\"your message\", \"your device token\", \"your push topic (optional)\")\n    ```\n\n3. Send the packet. Pushes are synchronous and return the notification with an\n   updated `:response` key.\n\n    ```elixir\n    Pigeon.APNS.push(n)\n    ```","ref":"apns-apple-ios.html#usage"},{"type":"extras","title":"Notification Struct - APNS (Apple iOS)","doc":"The contents of `payload` is what will be received on the iOS device. If\nupdating this field directly, use strings for your keys. It is recommended\nto use the convenience functions defined in *Notifications with Custom Data*.\n`expiration` is a UNIX epoch date in seconds (UTC). Passing a value of\n`0` expires the notification immediately and Apple will not attempt to\nredeliver it.\n\n  ```elixir\n  %Pigeon.APNS.Notification{\n    collapse_id: String.t() | nil,\n    device_token: String.t() | nil,\n    expiration: non_neg_integer | nil,\n    id: String.t() | nil,\n    payload: %{String.t() => String.t()},\n    response: atom,\n    topic: String.t() | nil\n  }\n  ```","ref":"apns-apple-ios.html#notification-struct"},{"type":"extras","title":"Generating Your Certificate and Key .pem - APNS (Apple iOS)","doc":"1. In Keychain Access, right-click your push certificate and\nselect _\"Export...\"_\n2. Export the certificate as `cert.p12`\n3. Click the dropdown arrow next to the certificate, right-click the private\nkey and select _\"Export...\"_\n4. Export the private key as `key.p12`\n5. From a shell, convert the certificate.\n\n     ```\n     openssl pkcs12 -legacy -clcerts -nokeys -out cert.pem -in cert.p12\n     ```\n\n6. Convert the key. Be sure to set a PEM pass phrase here. The pass phrase must be 4 or more characters in length or this will not work. You will need that pass phrase added here in order to remove it in the next step.\n\n     ```\n     openssl pkcs12 -legacy -nocerts -out key.pem -in key.p12\n     ```\n\n7. Remove the PEM pass phrase from the key.\n\n     ```\n     openssl rsa -in key.pem -out key_unencrypted.pem\n     ```\n\n8. `cert.pem` and `key_unencrypted.pem` can now be used as the cert and key\nin `Pigeon.push`, respectively. Set them in your `config.exs`","ref":"apns-apple-ios.html#generating-your-certificate-and-key-pem"},{"type":"extras","title":"Notifications with Custom Data - APNS (Apple iOS)","doc":"Notifications can contain additional information in `payload`. (e.g. setting\nbadge counters or defining custom sounds)\n\n  ```elixir\n  import Pigeon.APNS.Notification\n  n = Pigeon.APNS.Notification.new(\"message\", \"device token\", \"push topic\")\n  |> put_badge(5)\n  |> put_sound(\"default\")\n  |> put_content_available\n  |> put_mutable_content\n  |> put_category(\"category\")\n  ```\n  \nUsing a more complex `alert` dictionary?\n\n  ```elixir\n  n\n  |> put_alert(%{\n    \"title\" => \"alert title\",\n    \"body\" => \"alert body\"\n  })\n  ```\n  \nDefine custom payload data like so:\n\n  ```elixir\n  n\n  |> put_custom(%{\"your-custom-key\" => %{\n      \"custom-value\" => 500\n    }})\n  ```","ref":"apns-apple-ios.html#notifications-with-custom-data"},{"type":"extras","title":"Custom Worker Connections - APNS (Apple iOS)","doc":"Multiple APNS worker connections can be configured simultaneously.\nUseful for supporting multiple apps and/or certificates at once.\n\n  ```elixir\n  config :pigeon, :apns,\n    default: %{\n      cert: \"cert.pem\",\n      key: \"key_unencrypted.pem\",\n      mode: :dev\n    },\n    custom_worker: %{\n      cert: \"another_cert.pem\",\n      key: \"another_key_unencrypted.pem\",\n      mode: :prod\n    }\n  ```\n\nSend pushes with a `to` option in your second parameter.\n\n  ```elixir\n  n = Pigeon.APNS.Notification.new(\"message\", \"device token\", \"push topic\")\n  Pigeon.APNS.push(n, to: :custom_worker)\n  ```\n\nYou can also start connections manually.\n\n  ```elixir\n  iex> {:ok, pid} = Pigeon.APNS.start_connection(cert: \"cert.pem\",\n  ...> key: \"key.pem\", mode: :dev)\n  iex> Pigeon.APNS.push(notif, to: pid)\n\n  iex> Pigeon.APNS.start_connection(cert: \"cert.pem\", key: \"key.pem\",\n  ...> mode: :dev, name: :custom)\n  iex> Pigeon.APNS.push(notif, to: :custom)\n  ```","ref":"apns-apple-ios.html#custom-worker-connections"},{"type":"extras","title":"Asynchronous Pushing - APNS (Apple iOS)","doc":"1. Pass an `on_response` option with an anonymous function in your\nsecond parameter.\n\n    ```elixir\n    n = Pigeon.APNS.Notification.new(\"message\", \"device token\", \"push topic\")\n    Pigeon.APNS.push(n, on_response: fn(x) -> IO.inspect(x) end)\n    ```\n\n2. Responses return a notification with an updated `:response` key. \n   You could handle responses like so:\n\n    ```elixir\n    handler = fn(%Pigeon.APNS.Notification{response: response}) ->\n      case response do\n        :success ->\n          Logger.debug \"Push successful!\"\n        :bad_device_token ->\n          Logger.error \"Bad device token!\"\n        _error ->\n          Logger.error \"Some other error happened.\"\n      end\n    end\n\n    n = Pigeon.APNS.Notification.new(\"message\", \"device token\", \"push topic\")\n    Pigeon.APNS.push(n, on_response: handler)\n    ```","ref":"apns-apple-ios.html#asynchronous-pushing"},{"type":"extras","title":"Error Responses - APNS (Apple iOS)","doc":"*Taken from [APNS Provider API](https://developer.apple.com/library/content/documentation/NetworkingInternet/Conceptual/RemoteNotificationsPG/CommunicatingwithAPNs.html#//apple_ref/doc/uid/TP40008194-CH11-SW17)*\n\n|Reason                               |Description                   |\n|-------------------------------------|------------------------------|\n|`:bad_collapse_id`\t\t\t\t\t\t\t    \t|The collapse identifier exceeds the maximum allowed size|\n|`:bad_device_token`                  |The specified device token was bad. Verify that the request contains a valid token and that the token matches the environment.|\n|`:bad_expiration_date`               |The apns-expiration value is bad.|\n|`:bad_message_id`                    |The apns-id value is bad.|\n|`:bad_priority`                      |The apns-priority value is bad.|\n|`:bad_topic`                         |The apns-topic was invalid.|\n|`:device_token_not_for_topic`        |The device token does not match the specified topic.|\n|`:duplicate_headers`                 |One or more headers were repeated.|\n|`:idle_timeout`                      |Idle time out.|\n|`:missing_device_token`              |The device token is not specified in the request :path. Verify that the :path header contains the device token.|\n|`:missing_topic`                     |The apns-topic header of the request was not specified and was required. The apns-topic header is mandatory when the client is connected using a certificate that supports multiple topics.|\n|`:payload_empty`                     |The message payload was empty.|\n|`:topic_disallowed`                  |Pushing to this topic is not allowed.|\n|`:bad_certificate`                   |The certificate was bad.|\n|`:bad_certificate_environment`       |The client certificate was for the wrong environment.|\n|`:expired_provider_token`\t\t\t     \t|The provider token is stale and a new token should be generated.|\n|`:forbidden`                         |The specified action is not allowed.|\n|`:invalid_provider_token`\t\t\t     \t|The provider token is not valid or the token signature could not be verified.|\n|`:invalid_push_type`     \t\t\t     \t|The apns-push-type value is invalid.|\n|`:missing_provider_token`\t\t\t     \t|No provider certificate was used to connect to APNs and Authorization header was missing or no provider token was specified.|\n|`:bad_path`                          |The request contained a bad :path value.|\n|`:method_not_allowed`                |The specified :method was not POST.|\n|`:unregistered`                      |The device token is inactive for the specified topic.|\n|`:payload_too_large`                 |The message payload was too large. The maximum payload size is 4096 bytes.|\n|`:too_many_provider_token_updates`\t  |The provider token is being updated too often.|\n|`:too_many_requests`                 |Too many requests were made consecutively to the same device token.|\n|`:internal_server_error`             |An internal server error occurred.|\n|`:service_unavailable`               |The service is unavailable.|\n|`:shutdown`                          |The server is shutting down.|","ref":"apns-apple-ios.html#error-responses"},{"type":"extras","title":"FCM (Android)","doc":"# FCM (Android)","ref":"fcm-android.html"},{"type":"extras","title":"Usage - FCM (Android)","doc":"1. Set your environment variables.\n\n    ```elixir\n    config :pigeon, :fcm,\n      fcm_default: %{\n        key: \"your_fcm_key_here\"\n      }\n    ```\n\n2. Create a notification packet. \n\n    ```elixir\n    msg = %{\"body\" => \"your message\"}\n    n = Pigeon.FCM.Notification.new(\"your device registration ID\", msg)\n    ```\n \n3. Send the packet. Pushes are synchronous and return the notification with\n   updated `:status` and `:response` keys. If `:status` is success, `:response`\n   will contain a keyword list of individual registration ID responses.\n\n    ```elixir\n    Pigeon.FCM.push(n)\n    ```","ref":"fcm-android.html#usage"},{"type":"extras","title":"Sending to Multiple Registration IDs - FCM (Android)","doc":"Pass in a list of registration IDs, as many as you want.\n\n  ```elixir\n  msg = %{\"body\" => \"your message\"}\n  n = Pigeon.FCM.Notification.new([\"first ID\", \"second ID\"], msg)\n  ```","ref":"fcm-android.html#sending-to-multiple-registration-ids"},{"type":"extras","title":"Notification Struct - FCM (Android)","doc":"```elixir\n  %Pigeon.FCM.Notification{\n    collapse_key: nil | String.t(),\n    dry_run: boolean,\n    message_id: nil | String.t(),\n    payload: %{...},\n    priority: :normal | :high,\n    registration_id: String.t() | [String.t(), ...],\n    response: [] | [{atom, String.t()}, ...], | atom,\n    restricted_package_name: nil | String.t(),\n    status: atom | nil,\n    time_to_live: non_neg_integer\n  }\n  ```","ref":"fcm-android.html#notification-struct"},{"type":"extras","title":"Notifications with Custom Data - FCM (Android)","doc":"FCM accepts both `notification` and `data` keys in its JSON payload. Set them like so:\n\n  ```elixir\n  notification = %{\"body\" => \"your message\"}\n  data = %{\"key\" => \"value\"}\n  Pigeon.FCM.Notification.new(\"registration ID\", notification, data)\n  ```\n\nor\n\n  ```elixir\n  Pigeon.FCM.Notification.new(\"registration ID\")\n  |> put_notification(%{\"body\" => \"your message\"})\n  |> put_data(%{\"key\" => \"value\"})\n  ```","ref":"fcm-android.html#notifications-with-custom-data"},{"type":"extras","title":"Handling Push Responses - FCM (Android)","doc":"1. Pass an optional anonymous function as your second parameter.\n\n    ```elixir\n    data = %{message: \"your message\"}\n    n = Pigeon.FCM.Notification.new(data, \"device registration ID\")\n    Pigeon.FCM.push(n, fn(x) -> IO.inspect(x) end)\n    {:ok, %Pigeon.FCM.Notification{...}}\n    ```\n\n2. Reponses return the notification with an updated response.\n\n    ```elixir\n    on_response = fn(n) ->\n      case n.status do\n        :success ->\n          bad_regids = FCM.Notification.remove?(n)\n          to_retry = FCM.Notification.retry?(n)\n          # Handle updated regids, remove bad ones, etc\n        :unauthorized ->\n          # Bad FCM key\n        error ->\n          # Some other error\n      end\n    end\n    \n    data = %{message: \"your message\"}\n    n = Pigeon.FCM.Notification.new(\"your device token\", data)\n    Pigeon.FCM.push(n, on_response: on_response)\n    ```","ref":"fcm-android.html#handling-push-responses"},{"type":"extras","title":"Error Responses - FCM (Android)","doc":"*Slightly modified from [FCM Server Reference](https://firebase.google.com/docs/cloud-messaging/http-server-ref#error-codes)*\n\n|Reason                           |Description                  |\n|---------------------------------|-----------------------------|\n|`:missing_registration`          |Missing Registration Token   |\n|`:invalid_registration`          |Invalid Registration Token   |\n|`:not_registered`                |Unregistered Device          |\n|`:invalid_package_name`          |Invalid Package Name         |\n|`:authentication_error`          |Authentication Error         |\n|`:mismatch_sender_id`            |Mismatched Sender            |\n|`:invalid_json`                  |Invalid JSON                 |\n|`:message_too_big`               |Message Too Big              |\n|`:invalid_data_key`              |Invalid Data Key             |\n|`:invalid_ttl`                   |Invalid Time to Live         |\n|`:unavailable`                   |Timeout                      |\n|`:internal_server_error`         |Internal Server Error        |\n|`:device_message_rate_exceeded`  |Message Rate Exceeded        |\n|`:topics_message_rate_exceeded`  |Topics Message Rate Exceeded |\n|`:unknown_error`                 |Unknown Error                |","ref":"fcm-android.html#error-responses"},{"type":"extras","title":"ADM (Amazon Android)","doc":"# ADM (Amazon Android)","ref":"adm-amazon-android.html"},{"type":"extras","title":"Usage - ADM (Amazon Android)","doc":"1. Set your environment variables.\n\n    ```elixir\n    config :pigeon, :adm,\n      client_id: \"your_oauth2_client_id_here\",\n      client_secret: \"your_oauth2_client_secret_here\"\n    ```\n\n2. Create a notification packet.\n\n    ```elixir\n    msg = %{ \"body\" => \"your message\" }\n    n = Pigeon.ADM.Notification.new(\"your device registration ID\", msg)\n    ```\n\n3. Send the packet.\n\n    ```elixir\n    Pigeon.ADM.push(n)\n    ```","ref":"adm-amazon-android.html#usage"},{"type":"extras","title":"Handling Push Responses - ADM (Amazon Android)","doc":"1. Pass an optional anonymous function as your second parameter.\n\n    ```elixir\n    data = %{ message: \"your message\" }\n    n = Pigeon.ADM.Notification.new(\"device registration ID\", data)\n    Pigeon.ADM.push(n, on_response: fn(x) -> IO.inspect(x) end)\n    ```\n\n2. Responses return a notification with an updated `:response` key.\n   You could handle responses like so:\n\n    ```elixir\n    on_response_handler = fn(x) ->\n      case x.response do\n        :success ->\n          # Push successful\n          :ok\n        :update ->\n          new_reg_id = x.updated_registration_id\n          # Update the registration ID in the database\n        :invalid_registration_id ->\n          # Remove the bad ID from the database\n        :unregistered ->\n          # Remove the bad ID from the database\n        error ->\n          # Handle other errors\n      end\n    end\n\n    data = %{ message: \"your message\" }\n    n = Pigeon.ADM.Notification.new(\"your registration id\", data)\n    Pigeon.ADM.push(n, on_response: on_response_handler)\n    ```","ref":"adm-amazon-android.html#handling-push-responses"},{"type":"extras","title":"Error Responses - ADM (Amazon Android)","doc":"*Taken from [Amazon Device Messaging docs](https://developer.amazon.com/public/apis/engage/device-messaging/tech-docs/06-sending-a-message)*\n\n|Reason                           |Description                      |\n|---------------------------------|---------------------------------|\n|`:invalid_registration_id`       |Invalid Registration Token       |\n|`:invalid_data`                  |Bad format JSON data             |\n|`:invalid_consolidation_key`     |Invalid Consolidation Key        |\n|`:invalid_expiration`            |Invalid expiresAfter value       |\n|`:invalid_checksum`              |Invalid md5 value                |\n|`:invalid_type`                  |Invalid Type header              |\n|`:unregistered`                  |App instance no longer available |\n|`:access_token_expired`          |Expired OAuth access token       |\n|`:message_too_large`             |Data size exceeds 6 KB           |\n|`:max_rate_exceeded`             |See Retry-After response header  |\n|`:unknown_error`                 |Unknown Error                    |","ref":"adm-amazon-android.html#error-responses"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"1.6.2 - Changelog","doc":"**Changed**\n\n- Relax Poison dependency to allow 5.x.\n- Support HTTPoison 2.0, dropping support for ~> 0.7.\n\n**Fixed**\n\n- Handle new PEM decode case for APNS cert keys with OpenSSL 3.0.","ref":"changelog.html#1-6-2"},{"type":"extras","title":"v1.6.1 - Changelog","doc":"**Security**\n\n- Bump minimum Kadabra to 0.6.0. This is a critical security update for CA certificate validation.","ref":"changelog.html#v1-6-1"},{"type":"extras","title":"v1.6.0 - Changelog","doc":"**Added**\n\n- JSON library made configurable. For backwards compatability, Poison is still a required dependency.\n  Override in your `config.exs`.\n\n  ```\n  config :pigeon, json_library: Jason\n  ```\n\n**Fixed**\n\n- Handle FCM single message_id's on topic pushes.","ref":"changelog.html#v1-6-0"},{"type":"extras","title":"v1.5.1 - Changelog","doc":"- Added APNS InvalidPushType error ([#172](https://github.com/codedge-llc/pigeon/pull/172)).\n- Fixed various typespecs ([#170](https://github.com/codedge-llc/pigeon/pull/170)).","ref":"changelog.html#v1-5-1"},{"type":"extras","title":"v1.5.0 - Changelog","doc":"- Bumped minimum Elixir version to 1.6\n- Raise `Pigeon.ConfigError` when booting invalid config structs.\n  See below for validated keys and error types.\n- `APNS.JWTConfig` now validates key p8 content before connecting.\n- Relaxed `gen_stage` dependency to allow `~> 1.0`\n\nValidated config keys:\n\n- `ADM.Config` - `:client_id`, `:client_secret`\n- `APNS.Config` - `:cert`, `:key`\n- `APNS.JWTConfig` - `:team_id`, `:key`, `:key_identifier`\n- `FCM.Config` - `:key`\n\nPossible error values:\n\n- `{:error, {:invalid, value}}`\n- `{:error, {:nofile, value}}`","ref":"changelog.html#v1-5-0"},{"type":"extras","title":"v1.4.0 - Changelog","doc":"- `apns-push-type` header support for iOS 13. An additional `:push_type` key has been\n  added to the `APNS.Notification` struct.","ref":"changelog.html#v1-4-0"},{"type":"extras","title":"v1.3.2 - Changelog","doc":"- Document workers configuration for run-time configuration of push workers.\n- Modify run-time configuration of push workers so that multiple (or no)\n  workers may be returned by the startup configuration.","ref":"changelog.html#v1-3-2"},{"type":"extras","title":"v1.3.1 - Changelog","doc":"- Joken dependency bumped to 2.1","ref":"changelog.html#v1-3-1"},{"type":"extras","title":"v1.3.0 - Changelog","doc":"- Support for FCM `content_available`, `mutable_content`, and `condition` keys\n- Set `priority` of APNS notifications\n- Joken dependency bumped to 2.0.1","ref":"changelog.html#v1-3-0"},{"type":"extras","title":"v1.2.4 - Changelog","doc":"- Fixed ADM handling of connection timeouts","ref":"changelog.html#v1-2-4"},{"type":"extras","title":"v1.2.3 - Changelog","doc":"- Fixed APNS, FCM and ADM error response parse crashes. Error responses not\n  listed in the documentation are returned as `:unknown_error`","ref":"changelog.html#v1-2-3"},{"type":"extras","title":"v1.2.2 - Changelog","doc":"- Fixed APNS handling of notification `expiration`\n- Added APNS support for `collapse_id`","ref":"changelog.html#v1-2-2"},{"type":"extras","title":"v1.2.1 - Changelog","doc":"- FCM notifications can now handle `time_to_live`, `collapse_key`, `restricted_package_name`\n  and `dry_run` keys","ref":"changelog.html#v1-2-1"},{"type":"extras","title":"v1.2.0 - Changelog","doc":"- Support for APNS JWT configuration\n- Bump `kadabra` dependency to `v0.4.2`","ref":"changelog.html#v1-2-0"},{"type":"extras","title":"v1.1.6 - Changelog","doc":"- Relax `gen_stage` dependency to `~> 0.12`\n- Bump `kadabra` dependency to `v0.3.7`","ref":"changelog.html#v1-1-6"},{"type":"extras","title":"v1.1.5 - Changelog","doc":"- Fix: relax `httpoison` dependency to allow `0.x` or `1.0`","ref":"changelog.html#v1-1-5"},{"type":"extras","title":"v1.1.4 - Changelog","doc":"- Fix: `:on_response` callbacks spawned as supervised task instead of running\n  in the `Worker` process\n- Fix: ADM token refresh failure returns updated notification instead of\n  error tuple","ref":"changelog.html#v1-1-4"},{"type":"extras","title":"v1.1.3 - Changelog","doc":"- More robust FCM/APNS backpressure\n- Bumped minimum Kadabra version to `v0.3.6`","ref":"changelog.html#v1-1-3"},{"type":"extras","title":"v1.1.2 - Changelog","doc":"- Auto-restart connections if max stream ID is reached\n- FCM/APNS Workers now use GenStage to queue pending pushes\n- Bumped minimum Kadabra version to `v0.3.5`","ref":"changelog.html#v1-1-2"},{"type":"extras","title":"v1.1.1 - Changelog","doc":"- Bumped minimum Kadabra version to `v0.3.4`","ref":"changelog.html#v1-1-1"},{"type":"extras","title":"v1.1.0 - Changelog","doc":"- Minimum requirements now Elixir v1.4 and OTP 19.2 (Kadabra bumped\n  to `v0.3.0`)\n- Startup worker configs. Create a functions that return config\n  structs and specify them your `config.exs` with\n\n```elixir\nconfig :pigeon, workers: [\n  {YourApp.Pigeon, :apns_config},\n  {YourApp.Pigeon, :fcm_config},\n  {YourApp.Pigeon, :adm_config},\n  ...\n]\n```\n\n**APNS**\n\n- `APNS.Config.config/1` renamed to `APNS.Config.new/1`\n- `APNS.push/2` tagged tuples done away with in favor of a `:response` key on\n  the notification.\n- Override push server endpoint with `:uri` option in `APNS.Config.new/1`\n- `:use_2197` renamed to `:port`\n- `:uri` config option for overriding push server endpoint\n- `:reconnect` now false by default\n\n**FCM**\n\n- `NotificationResponse` done away with in favor of a `:response` key\n  on `Notification`\n- Override push server endpoint with `:uri` and `:port` options\n  in `FCM.Config.new/1`\n- `:uri` and `:port` config options for overriding push server endpoint\n\n**ADM**\n\n- `ADM.Config.config/1` renamed to `ADM.Config.new/1`\n- `ADM.push/2` tagged tuples done away with in favor of a `:response` key on\n  the notification.\n- `ADM.start_connection/1` and `ADM.stop_connection/1` added","ref":"changelog.html#v1-1-0"},{"type":"extras","title":"v1.0.4 - Changelog","doc":"- Fix: removed connection pinging from FCM.Worker (`:ping_period` option\n  left in FCM config to not break API)","ref":"changelog.html#v1-0-4"},{"type":"extras","title":"v1.0.3 - Changelog","doc":"- Fixed proper handling of large FCM push batches","ref":"changelog.html#v1-0-3"},{"type":"extras","title":"v1.0.2 - Changelog","doc":"- Fixed FCM infinite `GOAWAY session_timed_out` loop","ref":"changelog.html#v1-0-2"},{"type":"extras","title":"v1.0.1 - Changelog","doc":"- Configurable `:ping_period` for FCM connections","ref":"changelog.html#v1-0-1"},{"type":"extras","title":"v1.0.0 - Changelog","doc":"- GCM migrated to FCM API (http2)\n- `GCM` modules renamed to `FCM`\n- Set priority of FCM notifications\n- `APNSWorker` and `ADMWorker` renamed to `APNS.Worker` and `ADM.Worker`\n- Disable auto-reconnect for APNS workers with `reconnect: false`\n- Removed Chatterbox http2 client adapter","ref":"changelog.html#v1-0-0"},{"type":"extras","title":"v0.13.0 - Changelog","doc":"- Configurable `:ping_period` for APNS connections","ref":"changelog.html#v0-13-0"},{"type":"extras","title":"v0.12.1 - Changelog","doc":"- Various `chatterbox` client adapter fixes","ref":"changelog.html#v0-12-1"},{"type":"extras","title":"V0.12.0 - Changelog","doc":"- Configurable `Pigeon.Http2.Client`. Currently supports `kadabra`\n  and `chatterbox`\n- `kadabra` bumped to `v0.2.0`","ref":"changelog.html#v0-12-0"},{"type":"extras","title":"v0.11.0 - Changelog","doc":"- APNS workers can be started and referenced with pids and/or atom names\n- Fix: Push `:name` option renamed to `:to`\n- Fix: GCM/ADM async pushes now use `spawn/1` instead of `Task.async/1`","ref":"changelog.html#v0-11-0"},{"type":"extras","title":"v0.10.3 - Changelog","doc":"- Fix: cleaned up Elixir v1.4 warnings","ref":"changelog.html#v0-10-3"},{"type":"extras","title":"v0.10.2 - Changelog","doc":"- Fix: poison dependency version made optionally `~> 2.0 or ~> 3.0`","ref":"changelog.html#v0-10-2"},{"type":"extras","title":"v0.10.1 - Changelog","doc":"- Fix: kadabra not started","ref":"changelog.html#v0-10-1"},{"type":"extras","title":"v0.10.0 - Changelog","doc":"- Migrated HTTP/2 client from `chatterbox` to `kadabra`\n- Support for ADM (Amazon Android) push\n- APNS pushes are now synchronous by default. For async pushes use the\n  new `on_response` option. GCM and ADM will have it in the next major release.\n- Bulk APNS pushing\n- Handling of multiple APNS worker connections with different configs\n- Re-implemented APNS socket pings to keep connections open\n- `:invalid_jSON` corrected to `:invalid_json`","ref":"changelog.html#v0-10-0"},{"type":"extras","title":"v0.9.2 - Changelog","doc":"- Fixed GCM error response atom conversion","ref":"changelog.html#v0-9-2"},{"type":"extras","title":"v0.9.1 - Changelog","doc":"- Fixed :eaddrinuse error when restarting Pigeon too quickly with\n  :apns_2197 enabled","ref":"changelog.html#v0-9-1"},{"type":"extras","title":"v0.9.0 - Changelog","doc":"- APNS topic made optional\n- APNS `put_mutable_content` helper function added\n- GCM can be configured on a per-push basis\n- Updated to use Macro.underscore","ref":"changelog.html#v0-9-0"},{"type":"extras","title":"v0.8.0 - Changelog","doc":"- Implemented Chatterbox as APNS HTTP2 client\n- APNS server responses now caught asynchronously\n- GCM support for `notification` and `data` payload keys\n  (`Pigeon.GCM.Notification.new` API changes)","ref":"changelog.html#v0-8-0"},{"type":"extras","title":"v0.7.0 - Changelog","doc":"- APNS cert/key configs can now either be a file path, full-text string,\n  or `{:your_app, \"path/to/file.pem\"}` (which looks in the `/priv` directory\n  of your app)\n- Fixed APNSWorker crash on `:ssl.send/2` timeout\n- Better error-handling for invalid APNS configs","ref":"changelog.html#v0-7-0"},{"type":"extras","title":"v0.6.0 - Changelog","doc":"- `Pigeon.APNS.Notification.new/3` returns `%Pigeon.APNS.Notification{}` struct\n- Configure APNS to use SSL port 2197 with `apns_2197: true` in\n  your `config.exs`\n- Error feedback symbols converted from `CamelCase` to `snake_case`\n- APNS expiration values supported with `expiration` key in\n  `%Pigeon.APNS.Notification{}`","ref":"changelog.html#v0-6-0"},{"type":"extras","title":"v0.5.2 - Changelog","doc":"- Fixed bug where APNSWorker would hang up if SSL connection failed. Now\n  retries the connection twice more before gracefully shutting down.","ref":"changelog.html#v0-5-2"},{"type":"extras","title":"v0.5.1 - Changelog","doc":"- GCM error responses return proper chunk of regstration IDs","ref":"changelog.html#v0-5-1"},{"type":"extras","title":"v0.5.0 - Changelog","doc":"- `Pigeon.GCM.Notification.new/2` returns `%Pigeon.GCM.Notification{}` struct\n- Multiple registration IDs allowed in `Pigeon.GCM.push/2`\n- Remove `:gcm_worker`","ref":"changelog.html#v0-5-0"}],"content_type":"text/markdown"}